
/**
 * BitSetter.java
 *
 *
 * Created: Sat Oct 04 13:31:02 2003
 *
 * @author Michael Attig 
 * @version 0.99
 */

import java.io.*;
import java.util.StringTokenizer;
import java.math.*;

public class BitSetter  {
  
  private String filename;
  private int numHashFunctions;
  private int numHashBits;
  private int numMBFBits;
  private int maxMsgLength;
  private int hashArray[][][];

  private static int mbfNum;
  private static int []valueBRAM;
  private static int []valueXOR;

  public BitSetter(String filename, int numMBFBits) {
    this.filename = filename;
    this.numMBFBits = numMBFBits;
    mbfNum = 0;
    parseFile();
    valueBRAM = new int[numHashFunctions];
    valueXOR = new int[numHashFunctions];
  }
  

  private void parseFile(){


    try{
      File f = new File(filename);
      FileInputStream fis = new FileInputStream(f);
      BufferedReader br = new BufferedReader(new InputStreamReader(fis));
      
      String l;
      
      while (!(l=br.readLine()).startsWith("CONSTANT ran")){
	if (l.startsWith("CONSTANT")){
	  StringTokenizer st = new StringTokenizer(l, " ;");
	  String tok = st.nextToken();  //CONSTANT
	  //System.out.println("tok is " + tok);
	  String varName = st.nextToken();  //variable name
	  tok = st.nextToken(); // :
	  tok = st.nextToken(); // INTEGER
	  tok = st.nextToken(); // :=
	  int value = Integer.parseInt(st.nextToken());
	  if (varName.equals("num_hash_functions"))
	    numHashFunctions = value;
	  else if (varName.equals("num_hash_bits"))
	    numHashBits = value;
	  else if (varName.equals("max_msg_length"))
	    maxMsgLength = value;
	  //System.out.println("Var Name = " + varName + "  value = "+value);
	}
      }
      // now l = CONSTANT ran 

      // hash functions, string length (in bytes), bytes
      hashArray = new int[numHashFunctions][maxMsgLength][8];

      //fill in table
      for (int h=numHashFunctions-1; h>=0; h--){
	l = br.readLine();
	l = br.readLine();
	
	for (int byteNum=maxMsgLength-1; byteNum>=0; byteNum--){
	  l = br.readLine();
	  StringTokenizer s = new StringTokenizer(l, "\"-");
	  String bitTok = s.nextToken(); //( x
	  bitTok = s.nextToken(); //first value
	  //System.out.println("Bit Tok is " + bitTok);

	  for (int bitNum=7; bitNum >=0; bitNum--){
	    hashArray[h][byteNum][bitNum] = Integer.parseInt(bitTok,16);
	    bitTok = s.nextToken(); //, x
	    bitTok = s.nextToken(); // next value
	    //System.out.println("Bit Tok is " + bitTok + " h="+h+" byteNum="+byteNum+" bitNum="+bitNum);    
	  }	  
	  
	}
	
	l = br.readLine();

      }

      fis.close();
      br.close();
    }catch(FileNotFoundException fnfe){
      System.out.println("File Not Found " + fnfe);
    }catch(IOException ioe){
      System.out.println("IOException in parseFile " + ioe); 
    }
  }

  public static void main (String args[]){

    if (args.length == 0){
      System.out.println("Usage: java BitSetter <table filename>.vhd <# MBFs, default 1>");
      System.exit(0);
    }else if (args.length == 1){
      String s = args[0];
      String extension = s.substring(s.length()-3,s.length());
      if (!extension.equals("vhd")){
	System.out.println("Usage: java BitSetter <table filename>.vhd <#MBFs, default 1>");
	System.exit(0);
      }else{   
	readCommand(new BitSetter(args[0],1));
	System.exit(0);
      }
    }else if (args.length == 2){
      String s = args[0];
      String extension = s.substring(s.length()-3,s.length());
      if (!extension.equals("vhd")){
	System.out.println("Usage: java BitSetter <table filename>.vhd <#MBFs, default 1>");
	System.exit(0); 
      }else{   
	readCommand(new BitSetter(args[0],Integer.parseInt(args[1])));
	System.exit(0);
      }
    }else{
      System.out.println("Usage: java BitSetter <table filename>.vhd <# MBFs, default 1>");
      System.exit(0);
    }
  }

  //continually try and read the command line
  public static void readCommand(BitSetter b){  

    try{
      BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
      String s;

      while( !(s=in.readLine()).equals("quit")){

	//System.out.println("Read line was ... " + s);

	// eventually would like this to determine length of string/hex value
	// use this to determine which table to use
	// right now, assuming everything is same length
	if (s.charAt(0) == '\"'){
	  
	  if (s.length()-2 > b.getMaxMsgLength() || 
	      s.charAt(s.length()-1) != '\"'){
	    System.out.println("Usage: \"<String>\" -- For Example \"Mike\"");
	    System.out.println("  Note: Your string must be less than " + b.getMaxMsgLength()+ " characters");
	  }else{    
	    String myString = s.substring(1,s.length()-1);
	    
	    String hexRep ="";

	    //first convert all characters into hex equivalent
	    for (int c=0; c<myString.length(); c++){
	      int ch = (int)myString.charAt(c);
	      hexRep += "" + Integer.toString(ch,16);
	    }
	    System.out.println("hexrep is " +hexRep);

	    BigInteger hexvalue = new BigInteger(hexRep,16);   


	    System.out.println("------------------------------------------------------------");
	    System.out.println("|  MBF Number   |  Bram #  |  Hash Function  | Bit Address |");
	    System.out.println("------------------------------------------------------------");
	    BigInteger mult = new BigInteger("2");

	    for (int i=0; i<b.getNumHashFunctions(); i++){ //hash function loop
	      int xorValue = 0;
	      BigInteger mask = new BigInteger("1");
	      //System.out.println("mask is " + mask);
	      for(int j=0; j<(b.getMaxMsgLength()); j++){ //byte loop
		for (int k=0; k<8; k++){ //bit loop
		  if ( (mask.and(hexvalue)).compareTo(BigInteger.ZERO) > 0)
		    xorValue = xorValue ^ b.getValueAt(i,j,k);
		  mask = mask.multiply(mult); // *=2;
		  //System.out.println("Mask here is " + mask);
		}
	      }
	      
	      if (i == 0 && b.getNumMBFBits() != 1){
		mbfNum = 0;
		for (int m=0; m<b.getNumMBFBits(); m++){
		  int theBit = 5*m;
		  double theMask = Math.pow(2.0,(double)theBit);
		  if ((xorValue & (int)theMask) >0)
		    mbfNum += (int)(Math.pow(2.0,m));
		}
	      }

	      int bramNum = ((b.getNumHashFunctions()/2)*mbfNum)+(i/2);
	      
	      String addr = Integer.toString(xorValue,16);
	
	      valueBRAM[i] = bramNum;
	      valueXOR[i] = xorValue;
	      
      
	      System.out.println("|      "+mbfNum+"        |    "+bramNum+"     |        "+(i+1)+"        |     x"+addr+"    |"); 
	      System.out.println("------------------------------------------------------------");    
	    }
	    b.formControlPacketData();

	  }   

	}else if (s.charAt(0) == 'x'){
	  // need to determine which MBF to use
	  // need to tell which bits, in which block ram to set
	  // depends on hash function # and also on input string
	  if ( (s.length()-1)/2 <= b.getMaxMsgLength()){

	    BigInteger hexvalue = new BigInteger(s.substring(1,s.length()),16);
	    //System.out.println("Input Value, in decimal is " + hexvalue);
	    
	    //want to output a table, followed by what to copy/paste into cell
	    // actually, i'm just going to form the control packet
	    // ____________________________________________________________________
	    // |   MBF Number   |   BRAM #    |   Hash Function #  |  Bit Address |
	    // ____________________________________________________________________
	    
	    System.out.println("------------------------------------------------------------");
	    System.out.println("|  MBF Number   |  Bram #  |  Hash Function  | Bit Address |");
	    System.out.println("------------------------------------------------------------");
	    BigInteger mult = new BigInteger("2");

	    for (int i=0; i<b.getNumHashFunctions(); i++){ //hash function loop
	      int xorValue = 0;
	      BigInteger mask = new BigInteger("1");
	      //System.out.println("mask is " + mask);
	      for(int j=0; j<(b.getMaxMsgLength()); j++){ //byte loop
		for (int k=0; k<8; k++){ //bit loop
		  if ( (mask.and(hexvalue)).compareTo(BigInteger.ZERO) > 0)
		    xorValue = xorValue ^ b.getValueAt(i,j,k);
		  mask = mask.multiply(mult); // *=2;
		  //System.out.println("Mask here is " + mask);
		}
	      }
	      
	      if (i == 0 && b.getNumMBFBits() != 1){
		mbfNum = 0;
		for (int m=0; m<b.getNumMBFBits(); m++){
		  int theBit = 5*m;
		  double theMask = Math.pow(2.0,(double)theBit);
		  if ((xorValue & (int)theMask) >0)
		    mbfNum += (int)(Math.pow(2.0,m));
		}
	      }

	      int bramNum = ((b.getNumHashFunctions()/2)*mbfNum)+(i/2);
	      
	      String addr = Integer.toString(xorValue,16);
	
	      valueBRAM[i] = bramNum;
	      valueXOR[i] = xorValue;
	      
      
	      System.out.println("|      "+mbfNum+"        |    "+bramNum+"     |        "+(i+1)+"        |     x"+addr+"    |"); 
	      System.out.println("------------------------------------------------------------");    
	    }
	    b.formControlPacketData();

	  }else{ //if input too long a message
	    System.out.println("Too many bytes - Please insert number that is less than or equal to " +b.getMaxMsgLength()+ " bytes.");
	  }
	}else {
	  System.out.println("Usage: x<Hex Number> -- For Example xCDFE");
	  System.out.println(" -- or \"<String>\"    -- For Example \"Mike\"");
	}
      }
    }catch(IOException ioe){
      System.out.println("IOException " + ioe);
    }    
  }

  void formControlPacketData(){
    
    int length = numHashFunctions-1;
    String leng = Integer.toString(length,16);
    if (length < 15)
      leng = "0"+leng;
    
    String opcode = "74"+leng+"0000";

    System.out.println("45000048");
    System.out.println("00000000");
    System.out.println("40110000");
    System.out.println("7F000001");
    System.out.println("807F0001");
    System.out.println("0000BEEF");
    System.out.println("00340000");

    System.out.println(""+opcode);


    for (int i=0; i<numHashFunctions; i++){
      int bitSum = 1;//going to be setting that last bit

      int brams = valueBRAM[i];
      int addr  = valueXOR[i];

      brams = brams << 17; //left shift 17 bits
      addr  = addr  << 1;  //left shift 1 bit;

      bitSum += brams;
      bitSum += addr;
      
      String line = Integer.toString(bitSum,16);    
      //System.out.println("Line is " + line + " len " + line.length());
      int len = line.length();

      if (len != 8){
	for (int j=0; j<(8-len); j++){
	  line = "0" + line;
	}
      }
      System.out.println("" + line);


    }

  }

  


  public int getNumHashFunctions(){ return numHashFunctions; }
  public int getNumHashBits(){ return numHashBits; }
  public int getNumMBFBits() { return numMBFBits; }
  public int getMaxMsgLength() { return maxMsgLength; }
  public int getValueAt(int i, int j, int k){ return hashArray[i][j][k]; }

} // BitSetter
