LIBRARY std;
    USE std.textio.ALL;
LIBRARY ieee;
    USE ieee.std_logic_1164.ALL;
LIBRARY work;
    USE work.util_1164.ALL;
    USE work.std_logic_arith.ALL;

ENTITY tb IS
END tb;

ARCHITECTURE test OF tb IS

    CONSTANT addr_bits     : INTEGER := 17;
    CONSTANT data_bits     : INTEGER := 36;
    CONSTANT clk_start     : time    := 10 ns;
    CONSTANT clk_period    : time    := 10 ns;
    CONSTANT continue_time : time    := 10 ns;

    COMPONENT mt55l256l36p
        PORT (Dq    : INOUT STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
              Addr  : IN    STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);
              Lbo_n : IN    STD_LOGIC;
              Clk   : IN    STD_LOGIC;
              Cke_n : IN    STD_LOGIC;
              Ld_n  : IN    STD_LOGIC;
              Bwa_n : IN    STD_LOGIC;
              Bwb_n : IN    STD_LOGIC;
              Bwc_n : IN    STD_LOGIC;
              Bwd_n : IN    STD_LOGIC;
              Rw_n  : IN    STD_LOGIC;
              Oe_n  : IN    STD_LOGIC;
              Ce_n  : IN    STD_LOGIC;
              Ce2_n : IN    STD_LOGIC;
              Ce2   : IN    STD_LOGIC;
              Zz    : IN    STD_LOGIC);
    END COMPONENT;
    
    FOR ALL: mt55l256l36p USE ENTITY work.mt55l256l36p(behave);
    
    SIGNAL pDq       : STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0);
    SIGNAL pAddr     : STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);
    SIGNAL pLbo_n    : STD_LOGIC;
    SIGNAL pClk      : STD_LOGIC;
    SIGNAL pCke_n    : STD_LOGIC;
    SIGNAL pLd_n     : STD_LOGIC;
    SIGNAL pBwa_n    : STD_LOGIC;
    SIGNAL pBwb_n    : STD_LOGIC;
    SIGNAL pBwc_n    : STD_LOGIC;
    SIGNAL pBwd_n    : STD_LOGIC;
    SIGNAL pRw_n     : STD_LOGIC;
    SIGNAL pOe_n     : STD_LOGIC;
    SIGNAL pCe_n     : STD_LOGIC;
    SIGNAL pCe2_n    : STD_LOGIC;
    SIGNAL pCe2      : STD_LOGIC;
    SIGNAL pZz       : STD_LOGIC;
    SIGNAL stim_done : boolean := false;
    SIGNAL clk_done  : boolean := false;

    BEGIN
        U1 : mt55l256l36p
            PORT MAP (Dq => pDq, Addr => pAddr, Lbo_n => pLbo_n, Clk => pClk, Cke_n => pCke_n,
                      Ld_n => pLd_n, Bwa_n => pBwa_n, Bwb_n => pBwb_n, Bwc_n => pBwc_n, Bwd_n => pBwd_n,  
                      Rw_n => pRw_n, Oe_n => pOe_n, Ce_n => pCe_n, Ce2_n => pCe2_n, Ce2 => pCe2, Zz => pZz);

        stimulator : PROCESS

            FILE stim_file : text IS IN "vector.txt";
            VARIABLE l : line;
            VARIABLE time_var   : time;
            VARIABLE pDq_var    : INTEGER;
            VARIABLE pAddr_var  : INTEGER;
            VARIABLE pLbo_n_var : STD_LOGIC;
            VARIABLE pCke_n_var : STD_LOGIC;
            VARIABLE pLd_n_var  : STD_LOGIC;
            VARIABLE pBwa_n_var : STD_LOGIC;
            VARIABLE pBwb_n_var : STD_LOGIC;
            VARIABLE pBwc_n_var : STD_LOGIC;
            VARIABLE pBwd_n_var : STD_LOGIC;
            VARIABLE pRw_n_var  : STD_LOGIC;
            VARIABLE pOe_n_var  : STD_LOGIC;
            VARIABLE pCe_n_var  : STD_LOGIC;
            VARIABLE pCe2_n_var : STD_LOGIC;
            VARIABLE pCe2_var   : STD_LOGIC;
            VARIABLE pZz_var    : STD_LOGIC;

            VARIABLE pMa_var_vect : STD_LOGIC_VECTOR(addr_bits - 1 DOWNTO 0);
            CONSTANT HiZ : STD_LOGIC_VECTOR(data_bits - 1 DOWNTO 0) := (OTHERS => 'Z');

            BEGIN
                WHILE NOT ENDFILE (stim_file) LOOP
                    readline(stim_file, l);
                    IF l'length > 0 THEN
                        read(l, time_var);
                        read(l, pDq_var);   
                        read(l, pAddr_var); 
                        read(l, pLbo_n_var);
                        read(l, pCke_n_var);
                        read(l, pLd_n_var); 
                        read(l, pBwa_n_var);
                        read(l, pBwb_n_var);
                        read(l, pBwc_n_var);
                        read(l, pBwd_n_var);
                        read(l, pRw_n_var); 
                        read(l, pOe_n_var); 
                        read(l, pCe_n_var); 
                        read(l, pCe2_n_var);
                        read(l, pCe2_var);  
                        read(l, pZz_var);  
                        IF now > time_var THEN
                            ASSERT false
                                REPORT "Detected a time in the stim file that is in the past"
                                SEVERITY error;
                        ELSE
                            WAIT FOR time_var - now;
                                IF pDq_var = -100 THEN
                                    pDq <= HiZ;
                                ELSE  
                                    pDq <= CONV_STD_LOGIC_VECTOR(pDq_var, data_bits);
                                END IF;
                                pMa_var_vect := CONV_STD_LOGIC_VECTOR(pAddr_var, addr_bits);
                                pAddr  <= pMa_var_vect;
                                pLbo_n <= pLbo_n_var;
                                pCke_n <= pCke_n_var;
                                pLd_n  <= pLd_n_var;
                                pBwa_n <= pBwa_n_var;
                                pBwb_n <= pBwb_n_var;
                                pBwc_n <= pBwc_n_var;
                                pBwd_n <= pBwd_n_var;
                                pRw_n  <= pRw_n_var;
                                pOe_n  <= pOe_n_var;
                                pCe_n  <= pCe_n_var;
                                pCe2_n <= pCe2_n_var;
                                pCe2   <= pCe2_var; 
                                pZz    <= pZz_var;  
                        END IF;
                    END IF;
                END LOOP;
                
                ASSERT false
                    REPORT "End of stim file detected!"
                    SEVERITY note;
                stim_done <= true;
                WAIT;
        END PROCESS;
                               
        clock : PROCESS

        VARIABLE done_time : time;
            BEGIN
                pclk <= '0';
                WAIT for clk_start;
                    WHILE not stim_done loop
                        pclk <= '1';
                        WAIT for clk_period / 2;
                            pclk <= '0';
                        WAIT for clk_period / 2;
                    END LOOP;
                    done_time := now + continue_time;
                    WHILE now < done_time LOOP            --one last clock to finish last command
                        pclk <= '1';
                        WAIT for clk_period / 2;
                        pclk <= '0';
                        WAIT for clk_period / 2;
                    END LOOP;
                    ASSERT false
                        REPORT "Suspending clock activity"
                        SEVERITY note;
                    clk_done <= true;
                    WAIT;
        END PROCESS;

END test;
