-- file: IP_packets_in.vhd
-- simulation tool to capture ip packets from user application
-- Jing Lu jl1@arl.wustl.edu
-- Washington University
-- Applied Research Lab
-- Created: 8/12/03

-------------------------------------------------------------------------------
-- Modified 
--
-- 
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_textio.ALL;
USE std.textio.ALL;

ENTITY IP_packets_in IS
  generic (outfile : string := "output_IP.DAT");  -- output file
  PORT (
    clk : IN  STD_LOGIC;
    d_appl_in       : IN STD_LOGIC_VECTOR(31 DOWNTO 0); -- data to appl
    dataEn_appl_in  : IN std_logic;  -- data enable
    sof_appl_in     : IN std_logic;  -- start of frame
    soIP_appl_in    : IN std_logic;  -- start of IP header
    eof_appl_in     : IN std_logic;  -- end of frame
    sop_appl_in     : IN std_logic;  -- start of payload
    tca_appl_in     : OUT  std_logic);   -- congestion control

END IP_packets_in;


ARCHITECTURE behavioral OF IP_packets_in IS

  file out_file : text open write_mode is outfile;
  constant ip_packet  : STRING(8 DOWNTO 1) := "IPpacket";

BEGIN  -- behavioral
  tca_appl_in <= '1';          -- always accept data
  
  write_process : process
    variable line_out : LINE;
       
  begin
    wait until (clk'event and clk = '1');
      if soIP_appl_in = '1' then        
        write(line_out, ip_packet);
        writeline(out_file, line_out);
        while sof_appl_in = '0' loop
          if dataEn_appl_in = '1' then
            hwrite(line_out, d_appl_in);
            writeline(out_file, line_out);
          end if;
          wait until (clk'event and clk = '1');
        end loop; 
        if sof_appl_in = '1' and dataEn_appl_in = '1' then
          hwrite(line_out, d_appl_in);
          writeline(out_file, line_out);
        end if;       
      end if;      
  end process;
END behavioral;
