-- testbench.vhd


LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY testbench IS
  -- nothing
END testbench;

ARCHITECTURE simulation OF testbench IS
  component snort_app
  port (
    clk : in std_logic;
    reset_l : in std_logic;
    d_out_appl      : in STD_LOGIC_VECTOR(31 DOWNTO 0); -- data to appl
    dataEn_out_appl : in std_logic;  -- data enable
    sof_out_appl    : in std_logic;  -- start of frame
    soIP_out_appl   : in std_logic;  -- start of IP header
    eof_out_appl    : in std_logic;  -- end of frame
    sop_out_appl    : in std_logic;  -- start of payload
    tca_out_appl    : out  std_logic;   -- congestion control
    d_appl_in       : out STD_LOGIC_VECTOR(31 DOWNTO 0); -- data from appl
    dataEn_appl_in  : out std_logic;  -- data enable
    sof_appl_in     : out std_logic;  -- start of frame
    soIP_appl_in    : out std_logic;  -- start of IP header
    eof_appl_in     : out std_logic;  -- end of frame
    sop_appl_in     : out std_logic;  -- start of payload
    tca_appl_in     : in  std_logic;  -- congestion control
    -- To SRAM Device 0
    mod1_req                  : OUT STD_LOGIC;
    mod1_gr                   : IN STD_LOGIC;
    mod1_rw                   : OUT STD_LOGIC;
    mod1_addr                 : OUT STD_LOGIC_VECTOR(18 DOWNTO 0);
    mod1_d_in                 : IN STD_LOGIC_VECTOR(35 DOWNTO 0);
    mod1_d_out                : OUT STD_LOGIC_VECTOR(35 DOWNTO 0)
    );
  END component;
     
  component IP_packets_in
  generic (outfile : string := "output_IP.DAT");  -- output file
  PORT (
    clk : IN  STD_LOGIC;
    d_appl_in       : IN STD_LOGIC_VECTOR(31 DOWNTO 0); -- data from appl
    dataEn_appl_in  : IN std_logic;  -- data enable
    sof_appl_in     : IN std_logic;  -- start of frame
    soIP_appl_in    : IN std_logic;  -- start of IP header
    eof_appl_in     : IN std_logic;  -- end of frame
    sop_appl_in     : IN std_logic;  -- start of payload
    tca_appl_in     : OUT  std_logic);   -- congestion control
  END component;

  component IP_packets_out 
  generic (
    infile : string := "input_IP.DAT");  -- input file
  PORT (
    clk             : IN  STD_LOGIC;
    reset_l         : IN  STD_LOGIC;
    d_out_appl      : OUT STD_LOGIC_VECTOR(31 DOWNTO 0); -- data to appl
    dataEn_out_appl : OUT std_logic;  -- data enable
    sof_out_appl    : OUT std_logic;  -- start of frame
    soIP_out_appl   : OUT std_logic;  -- start of IP header
    eof_out_appl    : OUT std_logic;  -- end of frame
    sop_out_appl    : OUT std_logic;  -- start of payload
    tca_out_appl    : IN  std_logic);   -- congestion control
  END component;

  component clock
  port (
    clk : out std_logic);
  END component;

COMPONENT mt55l256l36p
  PORT (
    Dq    : INOUT STD_LOGIC_VECTOR (35 DOWNTO 0);  -- Data I/O
    Addr  : IN    STD_LOGIC_VECTOR (17 DOWNTO 0);  -- Address
    Lbo_n : IN    STD_LOGIC;                       -- Burst Mode
    Clk   : IN    STD_LOGIC;                       -- Clk
    Cke_n : IN    STD_LOGIC;                       -- Cke#
    Ld_n  : IN    STD_LOGIC;                       -- Adv/Ld#
    Bwa_n : IN    STD_LOGIC;                       -- Bwa#
    Bwb_n : IN    STD_LOGIC;                       -- BWb#
    Bwc_n : IN    STD_LOGIC;                       -- Bwc#
    Bwd_n : IN    STD_LOGIC;                       -- BWd#
    Rw_n  : IN    STD_LOGIC;                       -- RW#
    Oe_n  : IN    STD_LOGIC;                       -- OE# - Output Enable
    Ce_n  : IN    STD_LOGIC;                       -- CE#
    Ce2_n : IN    STD_LOGIC;                       -- CE2#
    Ce2   : IN    STD_LOGIC;                       -- CE2
    Zz    : IN    STD_LOGIC);                      -- Snooze Mode
END COMPONENT;

COMPONENT sram_interface
  PORT (clkin           : IN STD_LOGIC;
        resetin_l       : IN STD_LOGIC;
        -- SRAM Module I/O               -- 
        SRAM_Addr    : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);  -- 18 Address bits 
        SRAM_Data    : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
        SRAM_gw      : OUT   STD_LOGIC;  -- Global Write: LOW=write
        SRAM_ce      : OUT   STD_LOGIC;  -- Chip Enable: LOW=drive
        SRAM_pin84   : OUT   STD_LOGIC;  -- Address MSB; unused in 1Mb part
        -- Module 1 Interface
        mod1_req     : IN    std_logic ;
        mod1_gr      : OUT   std_logic ;
        mod1_rw      : IN    std_logic ;
        mod1_addr    : IN    std_logic_vector(18 DOWNTO 0) ;
        mod1_d_in    : OUT   std_logic_vector(35 DOWNTO 0) ;
        mod1_d_out   : IN    std_logic_vector(35 DOWNTO 0) ;
        -- Module 2 Interface
        mod2_req     : IN    std_logic ;
        mod2_gr      : OUT   std_logic ;
        mod2_rw      : IN    std_logic ;
        mod2_addr    : IN    std_logic_vector(18 DOWNTO 0) ;
        mod2_d_in    : OUT   std_logic_vector(35 DOWNTO 0) ;
        mod2_d_out   : IN    std_logic_vector(35 DOWNTO 0)
        );        
END COMPONENT ;

  signal clk : std_logic;
  signal reset_l : std_logic := '0';
  
  signal d_in : std_logic_vector(31 downto 0);
  signal dataEn_in : std_logic;
  signal sof_in, soip_in, sop_in, eof_in, tca_in: std_logic;

  signal d_out : std_logic_vector(31 downto 0);
  signal dataEn_out : std_logic;
  signal sof_out, soip_out, sop_out, eof_out, tca_out: std_logic;

  signal mod1_rw, mod1_gr, mod1_req     : STD_LOGIC;
  signal mod1_addr                      : STD_LOGIC_VECTOR(18 downto 0);
  signal mod1_d_in, mod1_d_out          : STD_LOGIC_VECTOR(35 downto 0);

  
-- SRAM dev 0
SIGNAL SRAM_Addr_dev0                   : STD_LOGIC_VECTOR(17 DOWNTO 0) ;
SIGNAL SRAM_Data_dev0                   : STD_LOGIC_VECTOR(35 DOWNTO 0) ;
SIGNAL SRAM_gw_dev0, SRAM_ce_dev0       : STD_LOGIC ;
SIGNAL SRAM_pin84_dev0                  : STD_LOGIC ;
-- SRAM dev 1
SIGNAL SRAM_Addr_dev1                   : STD_LOGIC_VECTOR(17 DOWNTO 0) ;
SIGNAL SRAM_Data_dev1                   : STD_LOGIC_VECTOR(35 DOWNTO 0) ;
SIGNAL SRAM_gw_dev1, SRAM_ce_dev1       : STD_LOGIC ;
SIGNAL SRAM_pin84_dev1                  : STD_LOGIC ;

BEGIN

  reset_l <= '1' after 120 ns;

  my_clock : clock
  port map(
    clk => clk);

  IP_IN : ip_packets_in
  generic map(outfile => "output_IP.DAT")  -- output file
  PORT map(
    clk => clk,
    d_appl_in       => d_in, -- data from appl
    dataEn_appl_in  => dataEn_in,  -- data enable
    sof_appl_in     => sof_in,  -- start of frame
    soIP_appl_in    => soip_in,  -- start of IP header
    eof_appl_in     => eof_in,  -- end of frame
    sop_appl_in     => sop_in,  -- start of payload
    tca_appl_in     => tca_in);
  
  IP_OUT : ip_packets_out
  generic map(infile => "input_IP.DAT")  -- input file
  PORT map(
    clk => clk,
    reset_l => reset_l,
    d_out_appl       => d_out, -- data to appl
    dataEn_out_appl  => dataEn_out,  -- data enable
    sof_out_appl     => sof_out,  -- start of frame
    soIP_out_appl    => soip_out,  -- start of IP header
    eof_out_appl     => eof_out,  -- end of frame
    sop_out_appl     => sop_out,  -- start of payload
    tca_out_appl     => tca_out); 

  my_snort: snort_app
  port map(
    clk              => clk,
    reset_l          => reset_l,
    d_out_appl       => d_out, -- data to appl
    dataEn_out_appl  => dataEn_out, -- data enable
    sof_out_appl     => sof_out,  -- start of frame
    soIP_out_appl    => soIP_out, -- start of IP header
    eof_out_appl     => eof_out, -- end of frame
    sop_out_appl     => sop_out, -- start of payload
    tca_out_appl     => tca_out, -- congestion control
    d_appl_in        => d_in,    -- data from appl
    dataEn_appl_in   => dataEn_in,-- data enable
    sof_appl_in      => sof_in, -- start of frame
    soIP_appl_in     => soIP_in, -- start of IP header
    eof_appl_in      => eof_in, -- end of frame
    sop_appl_in      => sop_in, -- start of payload
    tca_appl_in      => tca_in,   -- congestion control
    -- To SRAM Device 0
    mod1_req            => mod1_req,
    mod1_gr             => mod1_gr,
    mod1_rw             => mod1_rw,
    mod1_addr           => mod1_addr,
    mod1_d_in           => mod1_d_in,
    mod1_d_out          => mod1_d_out
    );    

    
SRAM0 : mt55l256l36p
PORT MAP (
  Dq    => SRAM_Data_dev0,            -- Data I/O
  Addr  => SRAM_Addr_dev0,            -- Address
  -- Note: Address bus really has 20 pins.
  -- Map SA0, SA1 to GND, and
  -- use remaining for word-size address.
  Lbo_n => '0',
  Clk   => clk,         
  Cke_n => '0',
  Ld_n  => '0',
  Bwa_n => '0',
  Bwb_n => '0',
  Bwc_n => '0',
  Bwd_n => '0',
  Rw_n  => SRAM_gw_dev0,              -- Read=1 / Write=0
  Oe_n  => '0',
  Ce_n  => SRAM_ce_dev0,              -- Chip enable
  Ce2_n => '0',
  Ce2   => '1',
  Zz    => '0');

SRAM1 : mt55l256l36p
  PORT MAP (
    Dq    => SRAM_Data_dev1,            -- Data I/O
    Addr  => SRAM_Addr_dev1,            -- Address
    -- Note: Address bus really has 20 pins.
    -- Map SA0, SA1 to GND, and
    -- use remaining for word-size address.
    Lbo_n => '0',
    Clk   => clk,
    Cke_n => '0',
    Ld_n  => '0',
    Bwa_n => '0',
    Bwb_n => '0',
    Bwc_n => '0',
    Bwd_n => '0',
    Rw_n  => SRAM_gw_dev1,              -- Read=1 / Write=0
    Oe_n  => '0',
    Ce_n  => SRAM_ce_dev1,              -- Chip enable
    Ce2_n => '0',
    Ce2   => '1',
    Zz    => '0');

sram_int0: sram_interface
  PORT MAP (
    clkin               => clk,
    resetin_l           => reset_l,
    -- I/O
    SRAM_Addr           => SRAM_Addr_dev0,
    SRAM_Data           => SRAM_Data_dev0,
    SRAM_gw             => SRAM_gw_dev0,
    SRAM_ce             => SRAM_ce_dev0,
    SRAM_pin84          => SRAM_pin84_dev0,
    -- mod1 interface
    mod1_req            => mod1_req,
    mod1_gr             => mod1_gr,
    mod1_rw             => mod1_rw,
    mod1_addr           => mod1_addr,
    mod1_d_in           => mod1_d_in,
    mod1_d_out          => mod1_d_out,
    -- mod2 interface
    mod2_req            => '0',
    mod2_gr             => OPEN,
    mod2_rw             => '1',
    mod2_addr           => (OTHERS => '0'),
    mod2_d_in           => OPEN,
    mod2_d_out          => (OTHERS => '0')
    ); 
  
  
END simulation;


